"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.traverse = traverse;
exports.filter = filter;
exports.which = which;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const child_process_1 = require("child_process");
const windows_1 = require("./windows");
const { platform, env } = process;
/**
 * Traverse File-system
 *
 * @param path - Root path to traverse
 */
function traverse(path) {
    return tslib_1.__asyncGenerator(this, arguments, function* traverse_1() {
        if (path === '/' && platform === 'win32') {
            for (const disk of (0, windows_1.getPartition)())
                yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(traverse(disk))));
            return yield tslib_1.__await(void 0);
        }
        for (let name of yield tslib_1.__await((0, fs_extra_1.readdir)(path))) {
            name = (0, path_1.join)(path, name);
            yield yield tslib_1.__await(name);
            try {
                if ((yield tslib_1.__await((0, fs_extra_1.stat)(name))).isDirectory())
                    yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(traverse(name))));
            }
            catch (error) {
                switch (error.code) {
                    case 'ENOENT':
                    case 'EPERM':
                    case 'EACCES':
                    case 'EBUSY':
                    case 'ELOOP':
                    case 'UNKNOWN':
                        continue;
                }
                throw error;
            }
        }
    });
}
/**
 * Iterator filter
 *
 * @param iterator
 * @param pattern  String pattern to match
 * @param count    Result count
 */
function filter(iterator_1, pattern_1) {
    return tslib_1.__asyncGenerator(this, arguments, function* filter_1(iterator, pattern, count = Infinity) {
        var _a, e_1, _b, _c;
        let index = 0;
        count = ~~count || Infinity;
        if (pattern && !(pattern instanceof RegExp))
            pattern = RegExp(pattern + '', 'i');
        try {
            for (var _d = true, iterator_2 = tslib_1.__asyncValues(iterator), iterator_2_1; iterator_2_1 = yield tslib_1.__await(iterator_2.next()), _a = iterator_2_1.done, !_a; _d = true) {
                _c = iterator_2_1.value;
                _d = false;
                const item = _c;
                if (!pattern || pattern.test(item))
                    if (index++ < count)
                        yield yield tslib_1.__await(item);
                    else
                        break;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = iterator_2.return)) yield tslib_1.__await(_b.call(iterator_2));
            }
            finally { if (e_1) throw e_1.error; }
        }
    });
}
function Shell_which(name) {
    try {
        return ((0, child_process_1.execSync)(`which ${name}`) + '').trim();
    }
    catch (error) {
        if (!('' + error.stdout + error.stderr).trim())
            return '';
    }
}
const MacAppPath = ['/Applications', `${env.HOME}/Applications`].filter(fs_extra_1.existsSync);
/**
 * @param name - Name (without extension name) of a executable file
 *
 * @return First matched path of a command
 */
async function which(name) {
    var _a, e_2, _b, _c, _d, e_3, _e, _f, _g, e_4, _h, _j;
    switch (platform) {
        case 'win32':
            for (const root of (0, windows_1.getAppFolder)())
                try {
                    for (var _k = true, _l = (e_2 = void 0, tslib_1.__asyncValues(filter(traverse(root), `\\\\${name}\\.exe$`, 1))), _m; _m = await _l.next(), _a = _m.done, !_a; _k = true) {
                        _c = _m.value;
                        _k = false;
                        const file = _c;
                        return file;
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (!_k && !_a && (_b = _l.return)) await _b.call(_l);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            break;
        case 'darwin': {
            const path = Shell_which(name);
            if (path)
                return path;
            for (const root of MacAppPath)
                try {
                    for (var _o = true, _p = (e_3 = void 0, tslib_1.__asyncValues(filter(traverse(root), `\\.app\\/Contents\\/MacOS\\/(\\w+\\W)?${name}$`, 1))), _q; _q = await _p.next(), _d = _q.done, !_d; _o = true) {
                        _f = _q.value;
                        _o = false;
                        const file = _f;
                        return file;
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (!_o && !_d && (_e = _p.return)) await _e.call(_p);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            break;
        }
        default: {
            const path = Shell_which(name);
            if (path)
                return path;
            try {
                for (var _r = true, _s = tslib_1.__asyncValues(filter(traverse('/opt'), `${name}$`, 1)), _t; _t = await _s.next(), _g = _t.done, !_g; _r = true) {
                    _j = _t.value;
                    _r = false;
                    const file = _j;
                    return file;
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (!_r && !_g && (_h = _s.return)) await _h.call(_s);
                }
                finally { if (e_4) throw e_4.error; }
            }
        }
    }
    return '';
}
